#ifndef _SHADOWLOG_INTERNAL_H
#define _SHADOWLOG_INTERNAL_H

extern const char *shadow_progname; /* Program name shown in error messages */
extern FILE *shadow_logfd;  /* file descriptor to which error messages are printed */

#endif /* _SHADOWLOG_INTERNAL_H */
