// SPDX-FileCopyrightText: 2024, Alejandro Colomar <alx@kernel.org>
// SPDX-License-Identifier: BSD-3-Clause


#include "config.h"

#include "string/strtok/stpsep.h"


extern inline char *stpsep(char *s, const char *delim);
