// SPDX-FileCopyrightText: 2025, Alejandro Colomar <alx@kernel.org>
// SPDX-License-Identifier: BSD-3-Clause


#include "config.h"

#include "string/ctype/strtoascii/strtolower.h"


extern inline char *strtolower(char *str);
