// SPDX-FileCopyrightText: 2024, Alejandro Colomar <alx@kernel.org>
// SPDX-License-Identifier: BSD-3-Clause


#include "config.h"

#include "string/strchr/strchrcnt.h"

#include <stddef.h>


extern inline size_t strchrcnt(const char *s, char c);
