// SPDX-FileCopyrightText: 2024-2025, Alejandro Colomar <alx@kernel.org>
// SPDX-License-Identifier: BSD-3-Clause


#include "config.h"

#include "string/strcmp/strcaseprefix.h"


extern inline const char *strcaseprefix_(const char *s, const char *prefix);
