// SPDX-FileCopyrightText: 2024, Alejandro Colomar <alx@kernel.org>
// SPDX-License-Identifier: BSD-3-Clause


#include "config.h"

#include "string/ctype/strisascii/strisprint.h"

#include <stdbool.h>


extern inline bool strisprint(const char *s);
