// SPDX-FileCopyrightText: 2024, Alejandro Colomar <alx@kernel.org>
// SPDX-License-Identifier: BSD-3-Clause


#include "config.h"

#include <stdbool.h>

#include "string/strcmp/streq.h"


extern inline bool streq(const char *s1, const char *s2);
